package com.sw.activemq.consumer;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.sw.activemq.producer.IMessageProducerService;
import com.sw.domain.TOrder;
import com.sw.service.SnatchMaskService;
import com.sw.service.TOrderService;
import lombok.extern.slf4j.Slf4j;
import org.apache.activemq.command.ActiveMQQueue;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Service;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import java.util.HashMap;
import java.util.Map;

@Service
@Slf4j
public class MessageConsumerService {

    @Autowired
    private SnatchMaskService snatchMaskService;
    @Autowired
    private IMessageProducerService iMessageProducerService;
    @Autowired
    private TOrderService tOrderService;

    @JmsListener(destination = "mask")
    public void receiveMessage(Message info) {    // 进行消息接收处理
        if (info instanceof MapMessage) {
            MapMessage objectMessage = (MapMessage) info;
            try {
                Integer mid = objectMessage.getInt("mid");
                Integer uid = objectMessage.getInt("uid");
                Map<String,Integer> map = new HashMap<>();
                map.put("uid",uid);
                map.put("mid",mid);
                snatchMaskService.doDecrAndCreateOrder(uid, mid);
                MsgPara msgPara = new MsgPara(uid+"",mid+"");
                //订单生成后发送延时消息，处理订单状态
				iMessageProducerService.delayMessage(new ActiveMQQueue("delay.mask.queue"),msgPara, (long) 60000);
            } catch (JMSException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 延时队列消费端
     * @param msgPara 消息
     */
    @JmsListener(destination = "delay.mask.queue")
    public void delayMask(MsgPara msgPara) {    // 进行消息接收处理
        if (null !=msgPara) {
            // 获取口罩id
            Integer mid =Integer.parseInt(msgPara.getMid());
            // 获取用户id
            Integer uid = Integer.parseInt(msgPara.getUid());
            QueryWrapper<TOrder> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("uid", uid);
            queryWrapper.eq("mid", mid);
            //查询订单
            TOrder order = tOrderService.getOne(queryWrapper);
            if (order != null) {
                if ("2".equals(order.getState())) { //判断订单是否已经处理
                    System.out.println("订单已处理");
                    return;
                } else if("1".equals(order.getState())) { //判断订单是否没有处理
                    order.setState("3");
                    //修改订单状态
                    tOrderService.updateById(order);
                    System.out.println("订单已过期");
                }
            }else{
                System.out.println("没有符合条件的订单");
            }
        }
    }
}
