package com.sw.activemq.producer.impl;

import com.sw.activemq.producer.IMessageProducerService;
import org.apache.activemq.ScheduledMessage;
import org.springframework.boot.autoconfigure.jms.JmsProperties;
import org.springframework.jms.core.JmsMessagingTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.jms.*;
import java.io.Serializable;
import java.util.Map;

@Service
public class MessageProducerServiceImpl implements IMessageProducerService {
	@Resource
	private JmsMessagingTemplate jmsMessagingTemplate;

	@Resource
	private Queue queue;

	@Override
	public void sendMessage(String message) {
		jmsMessagingTemplate.convertAndSend(this.queue, message);
	}

	@Override
	public void sendMessage(Map<String, Object> message) {
		jmsMessagingTemplate.convertAndSend(this.queue, message);
	}

	/**
	 * 描述 ：延时队列
	 * @param destination 队列
	 * @param data 消息
	 * @param time 延时时间
	 * @param <T>
	 */
	@Override
	public <T extends Serializable> void delayMessage(Destination destination, T data, Long time) {
		Connection connection =null;
		Session session =null;
		MessageProducer messageProducer =null;
		//获取连接工厂
		ConnectionFactory connectionFactory = jmsMessagingTemplate.getConnectionFactory();
		try {
			//获取连接
			connection = connectionFactory.createConnection();
			//开始
			connection.start();
			//获取事务
			session = connection.createSession(true,Session.AUTO_ACKNOWLEDGE);
			//创建一个消息队列
			messageProducer = session.createProducer(destination);
			messageProducer.setDeliveryMode(JmsProperties.DeliveryMode.PERSISTENT.getValue());
			ObjectMessage objectMessage = session.createObjectMessage( data);
			// 设置延迟时间
			objectMessage.setLongProperty(ScheduledMessage.AMQ_SCHEDULED_DELAY,time);
			// 发送消息
			messageProducer.send(objectMessage);
			//提交
			session.commit();
		} catch (JMSException e) {
			e.printStackTrace();
		}finally {
			try{
				if(messageProducer!=null){
					messageProducer.close();
				}
				if(session!=null){
					session.close();
				}
				if(connection!=null){
					connection.close();
				}

			}catch (Exception e){
				e.printStackTrace();
			}
		}

	}
}
