package com.sw;

import lombok.extern.slf4j.Slf4j;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.redisson.Redisson;
import org.redisson.api.RLock;
import org.redisson.api.RTimeSeries;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

/**
 * @author yegang
 * @create 2022-02-23 10:40
 **/
@Slf4j
@RunWith(SpringRunner.class)
@SpringBootTest
public class RedissonTest {
    @Autowired
    private RedissonClient redissonClient;
    private CountDownLatch count = new CountDownLatch(2);

    @Test
    public void lock() {
        RLock lock = redissonClient.getLock("mask-" + 1);
        new Thread(() -> {
            String threadName = Thread.currentThread().getName();
            log.info("线程：{} 正在尝试获取锁。。。", threadName);
            lock.lock(4000, TimeUnit.MILLISECONDS);
            try {
                Thread.sleep(3000);
                log.info("{}:业务执行完成", threadName);
                count.countDown();
            } catch (InterruptedException e) {
                e.printStackTrace();
            } finally {
                if(lock.isLocked()){
                    if(lock.isHeldByCurrentThread()){
                        lock.unlock();
                    }
                }
                log.info("线程：{}，释放了锁", threadName);
            }
        }).start();
        new Thread(() -> {
            String threadName = Thread.currentThread().getName();
            log.info("线程：{} 正在尝试获取锁。。。", threadName);
            lock.lock(4000, TimeUnit.MILLISECONDS);
            count.countDown();
            try {
                Thread.sleep(3000);
                log.info("{}:业务执行完成", threadName);
            } catch (InterruptedException e) {
                e.printStackTrace();
            } finally {
                if(lock.isLocked()){
                    if(lock.isHeldByCurrentThread()){
                        lock.unlock();
                    }
                }
                log.info("线程：{}，释放了锁", threadName);
            }
        }).start();
        try {
            count.await();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        log.info("子线程都已执行完毕，main函数可以结束了！");
    }

    @Test
    public void trylock() {
        RLock lock = redissonClient.getLock("mask-" + 1);
        new Thread(() -> {
            String threadName = Thread.currentThread().getName();
            log.info("线程：{} 正在尝试获取锁。。。", threadName);
            boolean b = false;
            try {
                b = lock.tryLock(4000, TimeUnit.MILLISECONDS);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            if (b) {
                try {

                    Thread.sleep(3000);
                    log.info("{}:业务执行完成", threadName);
                    count.countDown();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                } finally {
                    if(lock.isLocked()){
                        if(lock.isHeldByCurrentThread()){
                            lock.unlock();
                        }
                    }
                    log.info("线程：{}，释放了锁", threadName);
                }
            } else {
                log.info("{}:没有获取到锁，锁已被占用", threadName);
            }
        }).start();
        new Thread(() -> {
            String threadName = Thread.currentThread().getName();
            log.info("线程：{} 正在尝试获取锁。。。", threadName);
            try {
                if (lock.tryLock(4000, TimeUnit.MILLISECONDS)) {
                    try {

                        Thread.sleep(3000);
                        log.info("{}:业务执行完成", threadName);
                        count.countDown();
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    } finally {
                        if(lock.isLocked()){
                            if(lock.isHeldByCurrentThread()){
                                lock.unlock();
                            }
                        }
                        log.info("线程：{}，释放了锁", threadName);
                    }
                } else {
                    log.info("{}:没有获取到锁，锁已被占用", threadName);
                }
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }).start();
        try {
            count.await();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        log.info("子线程都已执行完毕，main函数可以结束了！");
    }
    @Test
    public void trylock2() {
        RLock lock = redissonClient.getLock("mask-" + 1);
        new Thread(() -> {
            String threadName = Thread.currentThread().getName();
            log.info("线程：{} 正在尝试获取锁。。。", threadName);
            boolean b = false;
            try {
                b = lock.tryLock(4000, TimeUnit.MILLISECONDS);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            if (b) {
                try {

                    Thread.sleep(6000);
                    log.info("{}:业务执行完成", threadName);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                } finally {
                    if(lock.isLocked()){
                        if(lock.isHeldByCurrentThread()){
                            count.countDown();
                            lock.unlock();
                        }
                    }
                    log.info("线程：{}，释放了锁", threadName);
                }
            } else {
                count.countDown();
                log.info("{}:没有获取到锁，锁已被占用", threadName);
            }
        }).start();
        new Thread(() -> {
            String threadName = Thread.currentThread().getName();
            log.info("线程：{} 正在尝试获取锁。。。", threadName);
            try {
                if (lock.tryLock(4000, TimeUnit.MILLISECONDS)) {
                    try {

                        Thread.sleep(6000);
                        log.info("{}:业务执行完成", threadName);

                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    } finally {
                        if(lock.isLocked()){
                            if(lock.isHeldByCurrentThread()){
                                count.countDown();
                                lock.unlock();
                            }
                        }
                        log.info("线程：{}，释放了锁", threadName);
                    }
                } else {
                    count.countDown();
                    log.info("{}:没有获取到锁，锁已被占用", threadName);
                }
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }).start();
        try {
            count.await();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        log.info("子线程都已执行完毕，main函数可以结束了！");
    }
    @Test
    public void timeout() {

        RLock lock = redissonClient.getLock("mask-" + 3);
        String threadName = Thread.currentThread().getName();
        log.info("线程：{} 正在尝试获取锁。。。", threadName);
        boolean b = false;
        try {
            b = lock.tryLock(4000, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        if (b) {
            try {

                Thread.sleep(5000);
                log.info("{}:业务执行完成", threadName);
            } catch (InterruptedException e) {
                e.printStackTrace();
            } finally {
                if(lock.isLocked()){
                    if(lock.isHeldByCurrentThread()){
                        lock.unlock();
                    }
                }
                log.info("线程：{}，释放了锁", threadName);
            }
        } else {
            log.info("{}:没有获取到锁，锁已被占用", threadName);
        }
}

}
